package alphablox;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.servlet.http.HttpServletRequest;

import java.io.*;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * HTMLFileParser.java
 */

/**
 *
 * Copyright 2002 AlphaBlox Corporation. All rights reserved.<BR>
 *
 * This utility class takes a URL, requests the content, and creates
 * the appropriate mime multiparts to represent the URL.
 *
 * @version $Revision: 1 $
 * @author  Dave Messink
 */

public class HTMLFileParser {
    private final String IMAGE_TAG_PATTER = "<img[^>]*>";
    private final String CSS_LINK_PATTERN = "<link[^>]*>";

    private Vector mimeParts = new Vector();
    private Map attachedFilenames = new HashMap();
    private int cidIndex = 0;
    private String imageSrcRegExp = "(src=(?:\"|')?)([^'\"]*)";
    private String linkHREFRegExp = "(href=(?:\"|')?)([^'\"]*)";

    public HTMLFileParser() {
    }

    public void setImageSrcRegExp(String imageSrcRegExp) {
        this.imageSrcRegExp = imageSrcRegExp;
    }

    public void setLinkHREFRegExp(String linkHREFRegExp) {
        this.linkHREFRegExp = linkHREFRegExp;
    }

    public void addBodyParts(EMailBean email) {
        Iterator partsList = mimeParts.iterator();

        while (partsList.hasNext())
            email.addBodyPart((MimeBodyPart) partsList.next());
    }

    public MimeBodyPart[] getBodyParts() {
        Iterator partsList = mimeParts.iterator();
        MimeBodyPart[] bodyParts = new MimeBodyPart[mimeParts.size()];
        int x = 0;

        while (partsList.hasNext())
            bodyParts[x++] = (MimeBodyPart) partsList.next();

        return bodyParts;
    }

    public void addAsAttachment(EMailBean emailBean, String filename) throws MessagingException, IOException {
        MimeMultipart multipart = new MimeMultipart("related");
        Iterator partsList = mimeParts.iterator();

        while (partsList.hasNext())
            multipart.addBodyPart((MimeBodyPart) partsList.next());

        Properties props = System.getProperties();
        props.put("mail.smtp.host", emailBean.getSMTPHost());

        Session session = Session.getDefaultInstance(props, null);

        Message message = new MimeMessage(session);
        message.setFrom();
        message.setSubject(filename);
        message.setSentDate(new Date());
        message.setHeader("MIME-Version", "1.0");
        message.setContent(multipart, multipart.getContentType());

        File mhtFile = File.createTempFile("temp",".mht");
        FileOutputStream stream = new FileOutputStream(mhtFile);
        message.writeTo(stream);
        stream.close();

        emailBean.attachFile(mhtFile, filename);
    }

    private MimeBodyPart getBodyPart() throws MessagingException {
        MimeMultipart multipart = new MimeMultipart("related");
        Iterator partsList = mimeParts.iterator();

        while (partsList.hasNext())
            multipart.addBodyPart((MimeBodyPart) partsList.next());

        MimeBodyPart bodyPart = new MimeBodyPart();
        bodyPart.setContent(multipart);
        bodyPart.setHeader("Content-Type", multipart.getContentType());

        return bodyPart;
    }

    public String getContents() throws MessagingException, IOException {
        MimeBodyPart bodyPart = getBodyPart();

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        bodyPart.writeTo(outputStream);

        return new String(outputStream.toByteArray(), "UTF-8");
    }

    public void parseHTMLFile(HttpServletRequest request, String urlPath) throws MessagingException, IOException {
        HTMLFile htmlFile = new HTMLFile(request, urlPath);
        String html = new String(htmlFile.getCharArray());
        String contentType = htmlFile.getContentType();

        try {
            html = parseImageTags(html, request);
            html = parseCSSLinks(html, request);
        } catch (Exception exc) {
            throw new RuntimeException(exc.getMessage());
        }

        // Create main html body part
        MimeBodyPart htmlPart = new MimeBodyPart();
        DataSource htmlSource = new HTMLDataSource(html, contentType);
        htmlPart.setDataHandler(new DataHandler(htmlSource));
        htmlPart.setHeader("Content-Type", contentType);

        mimeParts.insertElementAt(htmlPart, 0);
    }

    protected String parseImageTags(String html, HttpServletRequest request) throws IOException, MessagingException {
        return parseTags(html, request, IMAGE_TAG_PATTER, imageSrcRegExp);
    }

    protected String parseCSSLinks(String html, HttpServletRequest request) throws IOException, MessagingException {
        return parseTags(html, request, CSS_LINK_PATTERN, linkHREFRegExp);
    }

    private String parseTags(String html, HttpServletRequest request, String tagPattern, String linkPattern) throws IOException, MessagingException {
        int compileFlags = Pattern.CASE_INSENSITIVE | Pattern.MULTILINE;

        Pattern imageRegExp = Pattern.compile(tagPattern, compileFlags);
        Pattern srcRegExp = Pattern.compile(linkPattern, compileFlags);

        Matcher matchList = imageRegExp.matcher(String.valueOf(html));
        StringBuffer buffer = new StringBuffer();

        while (matchList.find()) {
            String imageTag = matchList.group();
            Matcher srcMatch = srcRegExp.matcher(imageTag);

            if (srcMatch.find()) {
                String cid = attachFile(request, srcMatch.group(2));

                matchList.appendReplacement(buffer, "");
                srcMatch.appendReplacement(buffer, "$1cid:" + cid);
                srcMatch.appendTail(buffer);
            } else
                matchList.appendReplacement(buffer, imageTag);
        }

        matchList.appendTail(buffer);

        return buffer.toString();
    }

    protected String attachFile(HttpServletRequest request, String urlPath)
            throws IOException, MessagingException {
        String cid = (String) attachedFilenames.get(urlPath);

        if (cid != null)
            return cid;

        // Get file contents and type
        HTMLFile file = new HTMLFile(request, urlPath);
        InputStream contents = file.getInputStream();
        String contentType = file.getContentType();

        // Create body part
        MimeBodyPart bodyPart = new MimeBodyPart();

        HTMLDataSource dataSource = new HTMLDataSource(contents, contentType);

        bodyPart.setDataHandler(new DataHandler(dataSource));

        // Handle special "known" encodings
        if (contentType.startsWith("image/"))
            bodyPart.setHeader("Content-Transfer-Encoding", "base64");
        else if (contentType.startsWith("text/css"))
            bodyPart.setHeader("Content-Transfer-Encoding", "quoted-printable");

        cid = "cid_" + (cidIndex++);
        bodyPart.setHeader("Content-Type", contentType);
        bodyPart.setHeader("Content-Location", urlPath);
        bodyPart.setHeader("Content-ID", cid);

        mimeParts.addElement(bodyPart);

        attachedFilenames.put(urlPath, cid);

        return cid;
    }

    public class HTMLDataSource implements DataSource {
        private byte[] data;	// html
        private String type;	// content-type

        /* Create a DataSource from a char array */
        public HTMLDataSource(String html, String type) throws IOException {
            this.type = type;
            data = html.getBytes("UTF-8");
        }

        public HTMLDataSource(InputStream inputStream, String type)
                throws IOException {
            this.type = type;

            ByteArrayOutputStream oStream = new ByteArrayOutputStream();
            BufferedInputStream iStream = new BufferedInputStream(inputStream);
            byte[] buffer = new byte[1024];
            int length;

            while ((length = iStream.read(buffer, 0, buffer.length)) > 0)
                oStream.write(buffer, 0, length);

            data = oStream.toByteArray();
        }

        /**
         * Return an InputStream for the data.
         * Note - a new stream must be returned each time.
         */
        public InputStream getInputStream() throws IOException {
            if (data == null)
                throw new IOException("no data");

            return new ByteArrayInputStream(data);
        }

        public OutputStream getOutputStream() throws IOException {
            throw new IOException("Not Supported");
        }

        public String getContentType() {
            return type;
        }

        public String getName() {
            return "dummy_name";
        }
    }
}
