/*
 * IBM Confidential
 * OCO Source Materials
 * 5724-L14
 *  Copyright IBM Corp. 2005
 * The source code for this program is not published or otherwise divested
 *   of its trade secrets, irrespective of what has been deposited with the
 *   U.S. Copyright Office.
 */

package dhtmlQueryBuilder;


import com.alphablox.blox.*;
import com.alphablox.blox.data.mdb.MDBMetaData;
import com.alphablox.blox.form.CheckBoxFormBlox;
import com.alphablox.blox.form.DataSourceSelectFormBlox;
import com.alphablox.blox.form.EditFormBlox;
import com.alphablox.blox.repository.DataSource;
import com.alphablox.blox.uimodel.BloxModel;
import com.alphablox.blox.uimodel.core.IModelDispatcher;
import com.alphablox.blox.uimodel.core.MessageBox;
import com.alphablox.util.DataException;
import com.alphablox.util.ILocalizedException;
import com.alphablox.util.Message;
import com.alphablox.util.TextFormat;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.util.Locale;

/*
 * QueryBuilderBean -- bean that encapsulates the application logic for the QueryBuilder
 *                     application.  The QueryBuilder application consists of the
 *                     following components:
 *
 * A presentblox for displaying the result set.
 * A connect dialog that consists of three formblox: one each for showing the list of
 *   data sources, the default user and and the password.
 * A multi-line text box for entering queries, which is represented as a formblox.
 * A series of static text boxes for displaying current logged in user, data source
 *   data source type and list of cubes.  These are currently represented by client
 *   side components.
 *
 * As part of the application logic, the QueryBuilderBean keeps these various components
 * in sync with one another.
 */

public class QueryBuilderBean {
	// Strings indicating adapter types.
	static public String DS_HYPERION = "Hyperion Essbase";
	static public String DS_HYPERION_EDS = "Essbase Enterprise Server";
	static public String DS_DB2FOROLAP = "IBM DB2 OLAP Server";
	static public String DS_DB2FOROLAP_DS = "IBM DB2 OLAP Server Deployment Services";
	static public String DS_MSAS = "MSOLAP.2";
	static public String DS_SSAS = "MSOLAP.3";
    static public String DS_SAP_BW = "MDrmSap.2";
	static private String DS_ALPHABLOXCUBE = "Alphablox cube database";
	static private String DS_SQLSERVER = "Microsoft SQL Server";
	static private String DS_ORACLE = "Oracle";
	// DB2 and Sybase return different types depending on the driver.
	static private String DS_DB2_TYPE3 = "DB2/6000";
	static private String DS_DB2_TYPE4 = "DB2/NT";
	static private String DS_SYBASE_1 = "Sybase SQL Server";
	static private String DS_SYBASE_2 = "Adaptive Server Enterprise";
	static private String DS_INFORMIX = "Informix";
	static private String DS_REDBRICK = "red brick";
	static private String DS_DERBY = "Apache Derby";
	static private String DS_DB2_AS400 = "AS/400";
    static private String DS_DB2_ZOS = "DB2";

	// Strings indicating catalog table name for relational databases.
	static public String SQLSERVER_CATALOG_TABLE_NAME = "TABLE_NAME";
	static public String DB2_CATALOG_TABLE_NAME = "TABNAME";
	static public String ORACLE_CATALOG_TABLE_NAME = "TNAME";

	private AppContext mAppContext;
	private DataBlox mDataBlox;
	// mCubeList should go once we figure out the best way to incorporate the
	// cube drop down box.
	private String mCubeList[];

	// Need a dispatcher for communicating with the client.
	private IModelDispatcher mDispatcher;

	// Formblox used in the application connect dialog
	private DataSourceSelectFormBlox mDataSource;
	private EditFormBlox mUserEdit;
	private EditFormBlox mPasswordEdit;
	private EditFormBlox mCatalogEdit;
	private EditFormBlox mSchemaEdit;
	private CheckBoxFormBlox mExecuteDefaultQuery;
	private CheckBoxFormBlox mAutoUpdateQuery;

	// we use this variable during connection.  If it is true, then we use
	// the user and password values supplied by the formblox.  If false,
	// we use the values in the data source.
	private boolean mUserChanged;

	// Maintains the query text.
	private EditFormBlox mQueryText;

	private AdminBlox mAdminBlox;

	public QueryBuilderBean() {
	}

	public boolean init(HttpServletRequest req, HttpServletResponse resp, AppContext appContext, DataBlox db, IModelDispatcher dispatcher) {
		try {
			this.setDataBlox(db);
			this.setDispatcher(dispatcher);
			this.mAppContext = appContext;

            BloxContext bloxContext = BloxContextFactory.getBloxContext(req, resp);

			if (mDataBlox.isConnected()) {
				setStatusOnClient(true, mAppContext.getString("CLIENT_MESSAGE_CONNECTED"));
			}

			mAdminBlox = new AdminBlox();
			mAdminBlox.init(bloxContext, "queryBuilderAdmin");

			// initialize the formblox.
			mUserEdit = new EditFormBlox();
			mUserEdit.init(bloxContext, "userEdit");
			mUserEdit.setVisible(false);
			mUserEdit.setThemeClass("inputDlg");

			UserEventListener userListener = new UserEventListener();
			userListener.init(this);
			mUserEdit.addFormEventListener(userListener);

			mPasswordEdit = new EditFormBlox();
			mPasswordEdit.init(bloxContext, "passwordEdit");
			mPasswordEdit.setVisible(false);
			mPasswordEdit.setMaskInput(true);
			mPasswordEdit.addFormEventListener(userListener);
			mPasswordEdit.setThemeClass("inputDlg");

			mUserChanged = false;

			mSchemaEdit = new EditFormBlox();
			mSchemaEdit.init(bloxContext, "schemaEdit");
			mSchemaEdit.setVisible(false);
			mSchemaEdit.setThemeClass("inputDlg");

			mCatalogEdit = new EditFormBlox();
			mCatalogEdit.init(bloxContext, "catalogEdit");
			mCatalogEdit.setVisible(false);
			mCatalogEdit.setThemeClass("inputDlg");

			mExecuteDefaultQuery = new CheckBoxFormBlox();
			mExecuteDefaultQuery.init(bloxContext, "defaultQueryCheckBox");
			mExecuteDefaultQuery.setFormElementName("executeDefaultQuery");
			mExecuteDefaultQuery.setVisible(false);
			mExecuteDefaultQuery.setChecked(false);
			mExecuteDefaultQuery.setThemeClass("inputDlg");

			mDataSource = new DataSourceSelectFormBlox();
			mDataSource.init(bloxContext, "dataSources");
			mDataSource.setVisible(false);
			mDataSource.setThemeClass("inputDlg");

			String initialDSString = mDataBlox.getDataSourceName();
			if ((initialDSString != null) && (!initialDSString.equals(""))) {
				// JGD is it possible to get a data source that isn't valid?
				mDataSource.setSelectedDataSourceName(initialDSString);
				DataSource initialDS = mAdminBlox.getDataSource(initialDSString);
				String[] userName = new String[]{initialDS.getUserName()};
				mUserEdit.setFormValues(userName);
			}
			DataSourceEventListener dsListener = new DataSourceEventListener();
			dsListener.init(mAdminBlox, mUserEdit, mPasswordEdit, mSchemaEdit, mCatalogEdit);
			mDataSource.addFormEventListener(dsListener);

			mQueryText = new EditFormBlox();
			mQueryText.init(bloxContext, "queryText");

			mAutoUpdateQuery = new CheckBoxFormBlox();
			mAutoUpdateQuery.init(bloxContext, "enableAutoUpdate");
			mAutoUpdateQuery.setVisible(true);
			mAutoUpdateQuery.setChecked(true);

			return true;
		} catch (Exception e) {
			mAppContext.getString("EXCEPTION_FAILED_TO_INITIALIZE", e.getMessage());
			setStatusOnClient(false, mAppContext.getString("CLIENT_MESSAGE_NOT_INITIALIZED"));
			return false;
		}

	}

	public EditFormBlox getQueryTextForm() {
		return mQueryText;
	}

	public DataBlox getDataBlox() {
		return mDataBlox;
	}

	public boolean isUserChanged() {
		return mUserChanged;
	}

	public void setUserChanged(boolean changed) {
		mUserChanged = changed;
	}

	public void setDataBlox(DataBlox dataBlox) {
		this.mDataBlox = dataBlox;
	}

	public void setDispatcher(IModelDispatcher dispatcher) {
		mDispatcher = dispatcher;
	}

// JGD getCubeList() and getCubeListAsString() can go once we incorporate the
// cube formblox.
	public String[] getCubeList() {
		return mCubeList;
	}

	public String getCubeListAsString() {
		String cubeListString = "";

		if (mCubeList != null) {
			for (int i = 0; i < mCubeList.length; ++i) {
				if (i > 0) cubeListString += ", ";
				cubeListString += mCubeList[i];
			}
		}
		return cubeListString;
	}

	protected void setClientConnectInfo() throws Exception {
		// The purpose of this method is to populate/update the client side static
		// text boxes with the current information.  Right now, we do this by
		// sending a series of client commands.  Eventually, we should have a server
		// side representation of the static text box and use that.

		mDispatcher.sendClientCommand("document.all.dataSourceName.innerHTML=\"" + mDataBlox.getDataSourceName() + "\"");
		mDispatcher.sendClientCommand("document.all.dataSourceType.innerHTML=\"" + mDataBlox.getMetaData().getDatabaseProductName() + "\"");
		mDispatcher.sendClientCommand("document.all.dataSourceUserName.innerHTML=\"" + mDataBlox.getUserName() + "\"");
		mDispatcher.sendClientCommand("document.all.schemaName.innerHTML=\"" + mDataBlox.getSchema() + "\"");
		mDispatcher.sendClientCommand("document.all.catalogName.innerHTML=\"" + mDataBlox.getCatalog() + "\"");
		setStatusOnClient(true, mAppContext.getString("CLIENT_MESSAGE_CONNECTED"));

		String cubeList = getCubeListAsString();
		cubeList = TextFormat.replace( cubeList, "'", "\\'" );
		mDispatcher.sendClientCommand("setCubeList('" + cubeList + "')");

		BloxModel model = mAutoUpdateQuery.getBloxModel();
		model.setDisabled(isRelational());
		model.changed();
	}

	public boolean connect() throws Exception {
		boolean returnStatus;

		try {
			if (mDataBlox.isConnected()) {
				mDataBlox.disconnect(true);
			}

			mDataBlox.setDataSourceName(mDataSource.getFormValue());
			if (isUserChanged()) {
				mDataBlox.setUserName(mUserEdit.getFormValue());
				mDataBlox.setPassword(mPasswordEdit.getFormValue());
			}
			setUserChanged(false);

			mDataBlox.setCatalog(mCatalogEdit.getFormValue());
			mDataBlox.setSchema(mSchemaEdit.getFormValue());
			mDataBlox.connect(false);

			// If the connect works, setup the cube list and execute the default query.
			// JGD setCubeList() call will go once we incorporate the cube form blox.
			setCubeList();
			setClientConnectInfo();
			setQueryText(getDefaultQuery(), false);
			if (mExecuteDefaultQuery.isChecked()) {
				executeQuery();
			}

			// setup the return values.
			returnStatus = true;
		} catch (DataBloxCannotConnectException e) {
			String error = mAppContext.getString("EXCEPTION_CANNOT_CONNECT", e.getCause().getMessage());
			System.out.println(error);
			setQueryText("", false);
			setStatusOnClient(false, mAppContext.getString("CLIENT_MESSAGE_NOT_CONNECTED"));
			MessageBox.message(mDispatcher, mAppContext.getString("STATUS_DIALOG_TITLE_CONNECTION"), error);
			returnStatus = false;
		} catch (ServerBloxException e) {
			String error = e.getCause().getMessage();
			System.out.println(error);
			setQueryText("", false);
			setStatusOnClient(false, mAppContext.getString("CLIENT_MESSAGE_NOT_CONNECTED"));
			MessageBox.message(mDispatcher, mAppContext.getString("STATUS_DIALOG_TITLE_CONNECTION"), error);
			returnStatus = false;
		} catch (Throwable e) {
			String error = e.getMessage();
			System.out.println(error);
			setQueryText("", false);
			setStatusOnClient(false, mAppContext.getString("CLIENT_MESSAGE_NOT_CONNECTED"));
			MessageBox.message(mDispatcher, mAppContext.getString("STATUS_DIALOG_TITLE_CONNECTION"), error);
			returnStatus = false;
		}
		return returnStatus;
	}

	public boolean executeQuery() {
		boolean returnStatus = true;

		try {
			if (!mDataBlox.isConnected()) {
				System.out.println(mAppContext.getString("EXCEPTION_CANNOT_EXECUTE_QUERY_NOT_CONNECTED"));
				setStatusOnClient(false, mAppContext.getString("CLIENT_MESSAGE_NOT_CONNECTED"));
				mDispatcher.sendClientCommand("alert('" + mAppContext.getString("NOT_CONNECTED_MESSAGE") + "')");
				returnStatus = false;
			}
            else {
				mDataBlox.clearResultSet();
				mDataBlox.setQuery(mQueryText.getFormValue());
				mDataBlox.updateResultSet();
				setStatusOnClient(true, mAppContext.getString("CLIENT_MESSAGE_QUERY_SUCCEEDED"));
			}
		}

        catch (DataException e) {
			try {
				mDataBlox.clearResultSet();
			}
            catch (ServerBloxException se) {
				String serverError = mAppContext.getString("EXCEPTION_SERVER_EXCEPTION", se.getCause().getMessage());
				System.out.println(serverError);
			}

			sendExceptionToDialog("EXCEPTION_DATA_EXCEPTION", e.getCause());
			returnStatus = false;
		}

        catch (ServerBloxException e) {
			try {
				mDataBlox.clearResultSet();
			} catch (ServerBloxException se) {
				String serverError = mAppContext.getString("EXCEPTION_SERVER_EXCEPTION", se.getCause().getMessage());
				System.out.println(serverError);
			}

            sendExceptionToDialog("EXCEPTION_SERVER_EXCEPTION", e.getCause());
			returnStatus = false;
		}

		catch (Throwable e) {
			try {
				mDataBlox.clearResultSet();
			} catch (ServerBloxException se) {
				String serverError = mAppContext.getString("EXCEPTION_SERVER_EXCEPTION", se.getCause().getMessage());
				System.out.println(serverError);
			}

            sendExceptionToDialog("EXCEPTION_SERVER_EXCEPTION", e);
			returnStatus = false;
		}
		return returnStatus;
	}

// JGD this method will go once we incorporate cube form blox.
	private void setCubeList() {
        try {
            String dbName = mDataBlox.getMetaData().getDatabaseProductName();

            // Right now, only ODBO data sources expose multiple cubes.  For Hyperion, there is
            // only one cube, and it's always named 'Cube1'.
            if (DS_MSAS.equalsIgnoreCase(dbName) ||
                    DS_SSAS.equalsIgnoreCase(dbName) ||
                    DS_ALPHABLOXCUBE.equalsIgnoreCase(dbName) ||
                    DS_SAP_BW.equalsIgnoreCase(dbName)) {
                MDBMetaData metaMdb = (MDBMetaData) mDataBlox.getMetaData();
                mCubeList = new String[metaMdb.getCubes().length];
                for (int i = 0; i < metaMdb.getCubes().length; ++i) {
                    mCubeList[i] = metaMdb.getCube(i).getName();
                }
            } else {
                mCubeList = null;
            }
        } catch (Exception e) {
            System.out.println(mAppContext.getString("EXCEPTION_ERROR_BUILDING_CUBE_LIST", e.getMessage()));
        }
    }

	public void refreshClientComponents() throws Exception {
		if (mDataBlox.isConnected()) {
			setClientConnectInfo();
		} else {
			setStatusOnClient(false, mAppContext.getString("CLIENT_MESSAGE_NOT_CONNECTED"));
		}
	}

	private void setStatusOnClient(boolean isOk, String status) {
		if (isOk) {
			mDispatcher.sendClientCommand("setStatus('true', '" + status + "')");
		} else {
			mDispatcher.sendClientCommand("setStatus('false', '" + status + "')");
		}
	}

	public boolean isRelational() throws Exception {
		if (mDataBlox.isConnected()) {
			int dbtype = mAdminBlox.getDataSource(mDataBlox.getDataSourceName()).getAdapterType();
			return (dbtype == DataSource.TYPE_RDB);
		}
		return false;
	}

	public boolean isAutoUpdateEnabled() throws Exception {
		if ((mDataBlox.isConnected()) &&
				(mAutoUpdateQuery.isChecked())
		) {
			return true;
		} else {
			return false;
		}
	}

	public String getDefaultQuery() throws Exception {
        String defaultQuery = null;

        if (!mDataBlox.isConnected()) {
            System.out.println(mAppContext.getString("EXCEPTION_CANNOT_SET_QUERY_NOT_CONNECTED"));
            setStatusOnClient(false, mAppContext.getString("CLIENT_MESSAGE_NOT_CONNECTED"));
        }

        else {
            String dbName = mDataBlox.getMetaData().getDatabaseProductName();

            if (DS_SQLSERVER.equalsIgnoreCase(dbName))
                defaultQuery = mAppContext.getString("SQLSERVER_DEFAULT_QUERY");
            else if (DS_ORACLE.equalsIgnoreCase(dbName))
                defaultQuery = mAppContext.getString("ORACLE_DEFAULT_QUERY");
            else if (dbName.indexOf(DS_INFORMIX) != -1)
                defaultQuery = mAppContext.getString("INFORMIX_DEFAULT_QUERY");
            else if (dbName.indexOf(DS_REDBRICK) != -1)
                defaultQuery = mAppContext.getString("REDBRICK_DEFAULT_QUERY");
            else if (dbName.indexOf(DS_DERBY) != -1)
                defaultQuery = mAppContext.getString("DERBY_DEFAULT_QUERY");
            else if (dbName.indexOf(DS_DB2_AS400) != -1)
                defaultQuery = mAppContext.getString("DB2_AS400_DEFAULT_QUERY");
            else if (dbName.equals(DS_DB2_ZOS))
                defaultQuery = mAppContext.getString("DB2_ZOS_DEFAULT_QUERY");
            else if ((DS_DB2_TYPE3.equalsIgnoreCase(dbName)) ||
                    (DS_DB2_TYPE4.equalsIgnoreCase(dbName))) {
                defaultQuery = mAppContext.getString("DB2_DEFAULT_QUERY");
            }
            else if ((DS_SYBASE_1.equalsIgnoreCase(dbName)) ||
                    (DS_SYBASE_2.equalsIgnoreCase(dbName))) {
                defaultQuery = mAppContext.getString("SYBASE_DEFAULT_QUERY");
            }
            else if (DS_MSAS.equalsIgnoreCase(dbName) || DS_SSAS.equalsIgnoreCase(dbName)) {
                if ((mCubeList != null) && (mCubeList.length > 0)) {
                    defaultQuery = mAppContext.getString("MSAS_DEFAULT_QUERY", mCubeList[0]);
                } else {
                    defaultQuery = "";
                    System.out.println(mAppContext.getString("WARNING_EMPTY_CUBELIST"));
                }
            }
            else if (DS_ALPHABLOXCUBE.equalsIgnoreCase(dbName)) {
                if ((mCubeList != null) && (mCubeList.length > 0)) {
                    defaultQuery = mAppContext.getString("ALPHABLOXCUBE_DEFAULT_QUERY", mCubeList[0]);
                } else {
                    defaultQuery = mAppContext.getString("ALPHABLOXCUBE_DEFAULT_QUERY", "<cubeName>");
                }
            }
            else if (DS_HYPERION.equalsIgnoreCase(dbName) ||
                    DS_DB2FOROLAP.equalsIgnoreCase(dbName) ||
                    dbName.startsWith(DS_DB2FOROLAP) ||
                    dbName.startsWith(DS_DB2FOROLAP_DS)) {
                defaultQuery = mAppContext.getString("HYPERION_DEFAULT_QUERY");
            }
            else if (DS_SAP_BW.equalsIgnoreCase(dbName)) {
                defaultQuery = mAppContext.getString("SAP_BW_DEFAULT_QUERY", mCubeList[0]);
            }
            else {
                defaultQuery = "";
                System.out.println(mAppContext.getString("WARNING_UNRECOGNIZED_DATASOURCE", dbName));
            }
        }
        return defaultQuery;
    }

	// if query is null, generate the query from the datablox.
	public void setQueryText(String query, boolean generate) throws Exception {
		String[] queryArray;

		if (!mDataBlox.isConnected()) {
			System.out.println(mAppContext.getString("EXCEPTION_CANNOT_SET_QUERY_NOT_CONNECTED"));
			setStatusOnClient(false, mAppContext.getString("CLIENT_MESSAGE_NOT_CONNECTED"));
			mDispatcher.sendClientCommand("alert('" + mAppContext.getString("NOT_CONNECTED_MESSAGE") + "')");
		} else {
			if (generate) {
				queryArray = new String[]{mDataBlox.generateQuery()};
			} else {
				queryArray = new String[]{query};
			}
			mQueryText.setFormValues(queryArray);
			mQueryText.changed();
		}
	}

	public void setQueryTextSize(int rows, int columns) throws Exception {
		mQueryText.setLines(rows);
		mQueryText.setCharactersPerLine(columns);
		mQueryText.changed();
	}

    private void sendExceptionToDialog(String bundleKey, Throwable t) {
        Throwable cause = t.getCause();
        String msg = null;
        String causedBy = "";
        Locale[] locales = new Locale[] {mAppContext.getPreferredLocale()};

        if (t instanceof ILocalizedException)
            msg = ((ILocalizedException)t).getLocalizedMessage(locales);
        else
            msg = t.getLocalizedMessage();

        if (cause != null) {
            if (cause instanceof ILocalizedException)
                causedBy = ((ILocalizedException)cause).getLocalizedMessage(locales);
            else
                causedBy = cause.getLocalizedMessage();

            if (!causedBy.equals(msg))
                causedBy = mAppContext.getString("EXCEPTION_CAUSED_BY", causedBy);
            else
                causedBy = "";
        }

        String error = mAppContext.getString(bundleKey, msg + " " + causedBy);

        new Message(Message.ERROR, "DHTMLQueryBuilder", t).send();
        setStatusOnClient(false, mAppContext.getString("CLIENT_MESSAGE_QUERY_FAILED"));
        MessageBox.message(mDispatcher, mAppContext.getString("STATUS_DIALOG_TITLE_QUERY_FAILED"), error);
    }
}
